-- Julia set forming mandelbrot.
-- By Martin Kotuliak

import qualified Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.IORef
import Data.Complex



size = 512 :: Float -- image will be square with height = width = 2 * size
zoom = 0.75 :: Float -- Smaller number -> bigger zoom
numOfSets = 10.0 :: Float -- size of square matrix

-- Random values for colors of julia sets
r = 79
g = 25
b = 91



main :: IO ()
main = do
  (progname,_) <- getArgsAndInitialize
  initialDisplayMode $= [DoubleBuffered]
  createWindow "Julia sets forming Mandelbrot"
  displayCallback $= (display)
  mainLoop

display :: IO ()
display = do
	clear [ColorBuffer]
	loadIdentity
	preservingMatrix (drawJulia)
	swapBuffers

drawJulia :: IO ()
drawJulia =
	renderPrimitive Points $ do
		mapM_ drawColoredPoint (allPoints)
	where
    	drawColoredPoint (x,y,c) = do
        	color c
        	vertex $ Vertex3 x y 0

allPoints :: [(GLfloat,GLfloat,Color3 GLfloat)]
allPoints = [ (realToFrac (x / size), realToFrac(y / size), colorFromValue $ julia x y) |
                  x <- [-size..size],
                  y <- [-size..size]]

colorFromValue :: Int -> Color3 GLfloat
colorFromValue n = 
	let
    	t :: Int -> Int -> GLfloat
    	t n i = fromIntegral((n * n * i - i + 50) `mod` 255) / 255
      -- pretty much random generator of colors.
  	in
    	Color3 (t n r) (t n g) (t n b)

divFloat :: Float -> Float -> Float
divFloat a b = fromIntegral (floor (a / b))

modFloat :: Float -> Float -> Float
modFloat a b = ((a / b) - (divFloat a b)) * b


julia :: Float -> Float -> Int
julia x y =
  let r = (((x / size + 1) `modFloat` (2 / numOfSets)) * numOfSets) - 1 
      i = (((y / size + 1) `modFloat` (2 / numOfSets)) * numOfSets) - 1
      c1 = ((((x / size + 1) `divFloat` (2 / numOfSets)) * (2 / numOfSets)) - 1) + ((2 / numOfSets) / 2) -- computing c so we can have multiple julia sets.
      c2 = ((((y / size + 1) `divFloat` (2 / numOfSets)) * (2 / numOfSets)) - 1) + ((2 / numOfSets) / 2)
  in
      f (2.0 * c1 :+ 2.0 * c2) (zoom * 2.0 * r :+ zoom * 2.0 * i) 128

-- function which does julia sets
f :: Complex Float -> Complex Float -> Int -> Int
f c z 0 = 0
f c z n = if (magnitude z > 2)
          then n
          else f c ((z * z) + c)  (n - 1)
